@@ -343,9 +343,7 @@ void ComplexHttpTask::check_response()
 	if (this->state == WFT_STATE_SYS_ERROR && this->error == ECONNRESET)
 	{
 		/* Servers can end the message by closing the connection. */
-		if (resp->is_header_complete() &&
-			!resp->is_keep_alive() &&
-			!resp->is_chunked() &&
+		if (resp->is_header_complete() && !resp->is_chunked() &&
 			!resp->has_content_length_header())
 		{
 			this->state = WFT_STATE_SUCCESS;
@@ -486,7 +484,8 @@ private:
 
 int ComplexHttpProxyTask::init_ssl_connection()
 {
-	SSL *ssl = __create_ssl(WFGlobal::get_ssl_client_ctx());
+	static SSL_CTX *ssl_ctx = WFGlobal::get_ssl_client_ctx();
+	SSL *ssl = __create_ssl(ssl_ctx_ ? ssl_ctx_ : ssl_ctx);
 	WFConnection *conn;
 
 	if (!ssl)
